@@ -6,13 +6,16 @@
 #' To find match ID, find the relevant matches on https://wwww.footywire.com
 #'
 #' @param ids A vector containing match id's to return. Can be a single value or vector of values.
+#' 
 #' @return Returns a data frame containing player match stats for each match ID
-#'
+#' 
+#' @export
+#' 
 #' @examples
 #' \dontrun{
 #' get_footywire_stats(ids = 5000:5100)
 #' }
-#' @export
+#'
 #' @importFrom magrittr %>%
 #' @importFrom rlang .data
 get_footywire_stats <- function(ids) {
@@ -102,7 +105,7 @@ update_footywire_stats <- function(check_existing = TRUE) {
     # dat_url <- "https://github.com/jimmyday12/fitzroy_data/blob/master/data-raw/player_stats/player_stats.rda" # nolint
     load_r_data <- function(fname) {
       tmp <- tempfile(fileext = ".rda")
-      utils::download.file(fname, tmp)
+      utils::download.file(fname, tmp, quiet = TRUE)
       
       load(tmp)
       unlink(tmp)
@@ -134,120 +137,69 @@ update_footywire_stats <- function(check_existing = TRUE) {
   }
 }
 
+DIGITS <- stringr::regex("round\\s+(\\d+)", ignore_case = TRUE)
 
-#' Helper function for \code{get_fixture,betting_data}
-#'
-#' Work out round number of each game from day and week.
-#' Games from Wednesday through Tuesday go in same Round.
-#'
-#' @param data_frame A data frame with match-level data and a Date column
-#' @importFrom magrittr %>%
-calculate_round <- function(data_frame) {
-  remove_bye_round_gaps <- function(gap_df) {
-    concat_round_groups <- function(Season, data, cumulative_week_lag) {
-      dplyr::mutate(
-        data,
-        Season = Season,
-        cumulative_week_lag = cumulative_week_lag
-      )
+parse_round_name <- function(max_regular_round_number) {
+  FINALS_WEEK <- stringr::regex("Finals\\s+Week\\s+(\\d+)", ignore_case = TRUE)
+  QUALIFYING_FINALS <- stringr::regex("qualifying", ignore_case = TRUE)
+  ELIMINATION_FINALS <- stringr::regex("elimination", ignore_case = TRUE)
+  # One bloody week in 2010 uses 'One' instead of '1' on
+  # https://www.footywire.com/afl/footy/afl_betting
+  FINALS_WEEK_ONE <- stringr::regex("Finals\\s+Week\\s+One", ignore_case = TRUE)
+  SEMI_FINALS <- stringr::regex("semi", ignore_case = TRUE)
+  PRELIMINARY_FINALS <- stringr::regex("preliminary", ignore_case = TRUE)
+  GRAND_FINAL <- stringr::regex("grand", ignore_case = TRUE)
+
+
+  return(
+    function(round_name) {
+      round_number <- stringr::str_match(round_name, DIGITS)[[2]]
+
+      if (!is.na(round_number)) {
+        return(round_number)
+      }
+
+      finals_week <- stringr::str_match(round_name, FINALS_WEEK)[[2]]
+
+      if (!is.na(finals_week)) {
+        # Betting data uses the format "YYYY Finals Week N" to label finals rounds
+        # so we can just add N to max round to get the round number
+        return(as.numeric(finals_week) + max_regular_round_number)
+      }
+
+      is_first_finals_week <- !is.na(stringr::str_match(round_name, QUALIFYING_FINALS)) ||
+        !is.na(stringr::str_match(round_name, ELIMINATION_FINALS)) ||
+        !is.na(stringr::str_match(round_name, FINALS_WEEK_ONE))
+
+      if (is_first_finals_week) {
+        return(max_regular_round_number + 1)
+      }
+
+      if (!is.na(stringr::str_match(round_name, SEMI_FINALS))) {
+        return(max_regular_round_number + 2)
+      }
+
+      if (!is.na(stringr::str_match(round_name, PRELIMINARY_FINALS))) {
+        return(max_regular_round_number + 3)
+      }
+
+      if (!is.na(stringr::str_match(round_name, GRAND_FINAL))) {
+        return(max_regular_round_number + 4)
+      }
     }
+  )
+}
 
-    calculate_max_lag <- function(weeks_since_last_match) {
-      # Subtract 1 with a floor of zero, because we expect 0-1 weeks since
-      # last match and don't want to include those round lags when calculating
-      # how much to shift rounds due to bye weeks.
-      max(max(weeks_since_last_match) - 1, 0)
-    }
+calculate_round_number <- function(round_names) {
+  max_regular_round_number <-  round_names %>%
+    stringr::str_match_all(., DIGITS) %>%
+    unlist(.) %>%
+    as.numeric(.) %>%
+    max(., na.rm = TRUE)
 
-    calculate_bye_week_lag <- function(season_data_frame) {
-      season_data_frame %>%
-        tidyr::nest(data = -c(.data$Round)) %>%
-        .$data %>%
-        # Expand max lag to have same length as match count.
-        purrr::map(~ rep.int(calculate_max_lag(.x$weeks_since_last_match), nrow(.x))) %>%
-        purrr::accumulate(~ max(.x) + .y) %>%
-        unlist
-    }
-
-    gap_df %>%
-      dplyr::mutate(
-        weeks_since_last_match = .data$Round - dplyr::lag(.data$Round, default = 0)
-      ) %>%
-      tidyr::nest(data = -c(.data$Season)) %>%
-      dplyr::mutate(
-        cumulative_week_lag = purrr::map(.data$data, calculate_bye_week_lag)
-      ) %>%
-      purrr::pmap(., concat_round_groups) %>%
-      dplyr::bind_rows(.) %>%
-      dplyr::mutate(Round = (.data$Round - .data$cumulative_week_lag)) %>%
-      dplyr::select(-c(.data$weeks_since_last_match, .data$cumulative_week_lag))
-  }
-
-  # Special cases where week counting doesn't work
-  fix_incorrect_rounds <- function(incorrect_df) {
-    round_df <- data.frame(incorrect_df)
-
-    # 2018 Collingwood/Essendon: Unlike default, this Wednesday match belongs
-    # to previous round (i.e. it's the end of the round, not the beginning)
-    round_five <- 5
-    # Need to use date for filter, because betting data doesn't include time
-    round_indices_to_fix <- lubridate::date(round_df$Date) == "2018-04-25"
-    round_df$Round[round_indices_to_fix] <- round_five
-
-    # 2012-2014: first round shifts round numbers for rest of season
-    round_one <- 1
-    round_indices_to_fix <- round_df$Season >= 2012 & round_df$Season <= 2014
-    shifted_rounds <- round_df$Round[round_indices_to_fix] - 1
-    round_df$Round[round_indices_to_fix] <- shifted_rounds
-    round_df$Round[round_df$Round == 0] <- round_one
-
-    # Round 13, 2010 and Round 18, 2014 each last two weeks, so we need to shift
-    # all subsequent rounds down by one
-    round_indices_to_fix <- (round_df$Round > 13 & round_df$Season == 2010) |
-      (round_df$Round > 18 & round_df$Season == 2014)
-    shifted_rounds <- round_df$Round[round_indices_to_fix] - 1
-    round_df$Round[round_indices_to_fix] <- shifted_rounds
-
-    # The 2010 Grand Final was replayed a week after the initial draw.
-    # AFLTables labels both matches as being in round 26, so we'll do the same
-    # here
-    round_twenty_six <- 26
-    round_indices_to_fix <- round_df$Round >= 26 & round_df$Season == 2010
-    round_df$Round[round_indices_to_fix] <- round_twenty_six
-
-    round_df
-  }
-
-  calculate_round_by_week <- function(roundless_df) {
-    sunday <- 1
-    tuesday <- 3
-
-    round_df <- roundless_df %>%
-      dplyr::mutate(
-        Season = lubridate::year(.data$Date),
-        week_count = lubridate::epiweek(.data$Date),
-        day_of_week = lubridate::wday(.data$Date),
-        Round = ifelse(
-          dplyr::between(.data$day_of_week, sunday, tuesday),
-          .data$week_count - 1,
-          .data$week_count
-        )
-      )
-
-    min_round <- round_df %>%
-      dplyr::group_by(.data$Season) %>%
-      dplyr::summarise(min_round = min(.data$Round))
-
-    round_df %>%
-      dplyr::left_join(., min_round, by = 'Season') %>%
-      dplyr::mutate(Round = as.integer(.data$Round - .data$min_round + 1)) %>%
-      dplyr::select(-c(.data$week_count, .data$day_of_week, .data$min_round))
-  }
-
-  round_df <- data_frame %>%
-    calculate_round_by_week(.) %>%
-    remove_bye_round_gaps(.) %>%
-    fix_incorrect_rounds(.)
+  round_names %>%
+    purrr::map(., parse_round_name(max_regular_round_number)) %>%
+    unlist(.)
 }
 
 
@@ -291,10 +243,36 @@ get_fixture <- function(season = lubridate::year(Sys.Date()),
   url_fixture <- paste0("https://www.footywire.com/afl/footy/ft_match_list?year=", season) # nolint
   fixture_xml <- xml2::read_html(url_fixture)
 
+  prepend_rounds_to_match_rows <- function(cumulative_nodes, current_node) {
+    current_column <- cumulative_nodes$current_column %% 8
+    is_round_node <- stringr::str_detect(current_node, stringr::regex("Round \\d+|Final", ignore_case = TRUE))
+
+    if (is_round_node) {
+      current_round <- current_node
+      nodes_to_append <- c(current_node)
+    } else {
+      current_round <- cumulative_nodes$current_round
+
+      if (current_column == 0) {
+        nodes_to_append <- c(current_round, current_node)
+      } else {
+        nodes_to_append <- c(current_node)
+      }
+    }
+
+    return(list(
+      current_round = current_round,
+      current_column = current_column + length(nodes_to_append),
+      nodes = c(cumulative_nodes$nodes, nodes_to_append)
+    ))
+  }
+
   # Get XML and extract text from .data
   games_text <- fixture_xml %>%
-    rvest::html_nodes(".data") %>%
-    rvest::html_text()
+    rvest::html_nodes(".data, .tbtitle") %>%
+    rvest::html_text() %>%
+    purrr::reduce(., prepend_rounds_to_match_rows, .init = list(current_column = 0)) %>%
+    .$nodes
 
 
   if (rlang::is_empty(games_text)) {
@@ -309,20 +287,24 @@ Check the following url on footywire
   }
 
   # Put this into dataframe format
-  games_df <- matrix(games_text, ncol = 7, byrow = TRUE) %>%
+  games_df <- matrix(games_text, ncol = 8, byrow = TRUE) %>%
     tibble::as_tibble() %>%
-    dplyr::select(.data$V1:.data$V3)
+    dplyr::select(.data$V1:.data$V4)
 
   # Update names
-  names(games_df) <- c("Date", "Teams", "Venue")
+  names(games_df) <- c("Round.Name", "Date", "Teams", "Venue")
 
   # Remove Bye & Match Cancelled
   games_df <- games_df %>%
     dplyr::filter(.data$Venue != "BYE" & .data$Venue != "MATCH CANCELLED")
 
   games_df <- games_df %>%
-    dplyr::mutate(Date = lubridate::ydm_hm(paste(season, .data$Date))) %>%
-    calculate_round(.)
+    dplyr::mutate(
+      Season = season,
+      Date = lubridate::ydm_hm(paste(season, .data$Date)),
+      Round = calculate_round_number(.data$Round.Name) %>% as.numeric(.)
+    ) %>%
+    dplyr::select(., !c('Round.Name'))
 
   # Fix names
   games_df <- games_df %>%
@@ -337,7 +319,7 @@ Check the following url on footywire
     )
 
   # Add season game number
-  games_df <- games_df %>% dplyr::mutate(Season.Game = dplyr::row_number())
+  games_df$Season.Game <- dplyr::row_number(games_df$Date)
 
   # Fix Teams
   # Uses internal replace teams function
@@ -381,24 +363,6 @@ get_footywire_betting_odds <- function(
                                        end_season = lubridate::year(Sys.Date())) {
   if (class(end_season) == "Date") format(end_season, "%Y")
 
-  raw_betting_col_names <- c(
-    "Date",
-    "Venue",
-    "blank_one",
-    "Team",
-    "Score",
-    "Margin",
-    "Win.Odds",
-    "Win.Paid",
-    "Line.Odds",
-    "colon",
-    "redundant_line_paid",
-    "Line.Paid",
-    "blank_two",
-    "blank_three",
-    "Season"
-  )
-
   rename_home_away_columns <- function(col_name) {
     paste0(
       ifelse(grepl("_home$", col_name), "Home.", "Away."),
@@ -427,7 +391,7 @@ get_footywire_betting_odds <- function(
     rows %>% purrr::map(~ c(row_padding(., max_row_length), .))
   }
 
-  clean_table_row <- function(table_row_html) {
+  extract_text <- function(table_row_html) {
     table_row_html %>%
       rvest::html_text(.) %>%
       stringr::str_split(., "\\n") %>%
@@ -438,6 +402,14 @@ get_footywire_betting_odds <- function(
       stringr::str_trim(.)
   }
 
+  fetch_valid_seasons <- function(){
+    page <- fetch_betting_odds_page(2010)
+    years <- page[[1]] %>% 
+            rvest::html_nodes("option") %>%
+      rvest::html_text() %>%
+      as.numeric()
+  }
+  
   extract_table_rows <- function(page_html, season) {
     data_table_row_selector <- "form table table table tr"
 
@@ -447,7 +419,7 @@ get_footywire_betting_odds <- function(
     if (length(table_rows) == 0) {
       warning(paste0(
         "Skipping ",
-        season, ", 
+        season, ",
                      because it doesn't have any data."
       ))
 
@@ -455,16 +427,32 @@ get_footywire_betting_odds <- function(
     }
 
     table_rows <- table_rows %>%
-      purrr::map(clean_table_row) %>%
-      purrr::keep(~ length(.x) > 1)
+      purrr::map(extract_text)
 
-    column_label_row <- table_rows[[1]]
+    append_rounds_to_match_rows <- function(cumulative_rows, current_row) {
+      ROUND_ROW <- stringr::regex("Round \\d+|Final", ignore_case = TRUE)
+      is_round_node <- all(stringr::str_detect(current_row, ROUND_ROW))
+
+      if (is_round_node) {
+        current_round <- current_row
+        rows <- cumulative_rows$rows
+      } else {
+        current_round <- cumulative_rows$current_round
+        rows <- c(cumulative_rows$rows, list(c(current_row, current_round)))
+      }
+
+      list(current_round = current_round, rows = rows)
+    }
+
+    column_label_row <- table_rows[[2]]
     is_column_label_row <- function(row) {
       length(row) == length(column_label_row) && all(row == column_label_row)
     }
 
     table_rows %>%
-      purrr::discard(is_column_label_row) %>%
+      purrr::discard(., ~ is_column_label_row(.x) || all(.x == "")) %>%
+      purrr::reduce(., append_rounds_to_match_rows, .init = list()) %>%
+      .$rows %>%
       normalize_row_length(.) %>%
       purrr::map(~ c(.x, as.character(season)))
   }
@@ -478,7 +466,95 @@ get_footywire_betting_odds <- function(
       list(., season)
   }
 
-  valid_start_season <- max(as.numeric(start_season), 2010)
+  convert_to_data_frame <- function(raw_season_data) {
+    raw_betting_col_names <- c(
+      "Date",
+      "Venue",
+      "blank_one",
+      "Team",
+      "Score",
+      "Margin",
+      "Win.Odds",
+      "Win.Paid",
+      "Line.Odds",
+      "colon",
+      "redundant_line_paid",
+      "Line.Paid",
+      "blank_two",
+      "blank_three",
+      "Round.Name",
+      "Season"
+    )
+
+    n_columns <- length(raw_betting_col_names)
+
+    raw_season_values <- unlist(raw_season_data)
+    season_values <- if (is.null(raw_season_values)) {
+      # Need two rows-worth of NAs to allow for Home and Away columns after pivoting
+      rep_len(NA, n_columns * 2)
+    } else {
+      raw_season_values
+    }
+
+    season_values %>%
+      matrix(
+        .,
+        ncol = n_columns,
+        byrow = TRUE,
+        dimnames = list(NULL, raw_betting_col_names)
+      ) %>%
+      as.data.frame(.) %>%
+      tidyr::fill(c(.data$Date, .data$Venue)) %>%
+      dplyr::mutate(
+        Date = lubridate::dmy(.data$Date),
+        Venue = as.character(.data$Venue),
+        Team = as.character(.data$Team),
+        Score = as.character(.data$Score) %>% as.numeric(.),
+        Margin = as.character(.data$Margin) %>%
+          stringr::str_replace_all(., "\\+", "") %>%
+          as.numeric(.),
+        Win.Odds = as.character(.data$Win.Odds) %>% as.numeric(.),
+        Win.Paid = as.character(.data$Win.Paid) %>% as.numeric(.),
+        Line.Odds = as.character(.data$Line.Odds) %>%
+          stringr::str_replace_all(., "\\+", "") %>%
+          as.numeric(.),
+        Line.Paid = as.character(.data$Line.Paid) %>% as.numeric(.),
+        Round = calculate_round_number(.data$Round.Name) %>% as.numeric(.),
+        Season = as.character(.data$Season) %>% as.numeric(.),
+        # Raw betting data has two rows per match: the top team is home
+        # and the bottom is away
+        Home.Away = ifelse(dplyr::row_number() %% 2 == 1, "home", "away"),
+        # We need a unique Match.ID to pivot rows correctly, because there are
+        # some duplicate date/venue combinations
+        Match.ID = ceiling(seq(dplyr::row_number()) / 2)
+      )
+  }
+
+  valid_seasons <- fetch_valid_seasons()
+  valid_end_season <- min(
+    as.numeric(end_season),
+    max(valid_seasons)
+  )
+
+  if (is.na(valid_end_season)) {
+    stop(paste0(valid_end_season, " couldn't be coerced to a valid year."))
+  }
+
+  if (as.numeric(end_season) > valid_end_season) {
+    warning(
+      paste0(
+        valid_end_season,
+        " is the last season for which betting data is available,
+        so ending in ",
+        valid_end_season,
+        " instead of ",
+        end_season
+      )
+    )
+  }
+
+  valid_start_season <- max(as.numeric(start_season), min(valid_seasons))
+  valid_start_season <- min(valid_start_season, valid_end_season)
 
   if (is.na(valid_start_season)) {
     stop(paste0(valid_start_season, " couldn't be coerced to a valid year."))
@@ -494,76 +570,28 @@ get_footywire_betting_odds <- function(
     )
   }
 
-  valid_end_season <- min(
-    as.numeric(end_season),
-    as.numeric(lubridate::year(Sys.Date()))
-  )
-
-  if (is.na(valid_end_season)) {
-    stop(paste0(valid_end_season, " couldn't be coerced to a valid year."))
+  
+  season_range <- valid_start_season:valid_end_season
+  season_range <- season_range[season_range %in% valid_seasons]
+  
+  if(length(season_range) < 1) {
+    rlang::warn("No valid seasons found")
+    return(NA)
   }
-
-  if (as.numeric(end_season) > valid_end_season) {
-    warning(
-      paste0(
-        valid_end_season,
-        " is the last season for which betting data is available, 
-        so ending in ",
-        valid_end_season,
-        " instead of ",
-        end_season
-      )
-    )
-  }
-
-  n_columns <- length(raw_betting_col_names)
-
-  raw_betting_values <- valid_start_season:valid_end_season %>%
+  
+  betting_dfs <- season_range %>%
     purrr::map(fetch_betting_odds_page) %>%
     purrr::map(., ~ do.call(extract_table_rows, .)) %>%
-    unlist()
+    purrr::map(convert_to_data_frame)
 
-  betting_values <- if (is.null(raw_betting_values)) {
-    # Need two rows-worth of NAs to allow for Home and Away columns after pivoting
-    rep_len(NA, n_columns * 2)
-  } else {
-    raw_betting_values
-  }
-
-  betting_values %>%
-    matrix(
-      .,
-      ncol = n_columns,
-      byrow = TRUE,
-      dimnames = list(NULL, raw_betting_col_names)
-    ) %>%
-    as.data.frame(.) %>%
+  betting_dfs %>%
+    dplyr::bind_rows(.) %>%
     dplyr::select(
       -c(
         "blank_one", "colon", "redundant_line_paid",
-        "blank_two", "blank_three"
+        "blank_two", "blank_three", "Round.Name"
       )
     ) %>%
-    tidyr::fill(c(.data$Date, .data$Venue)) %>%
-    dplyr::mutate(
-      Date = lubridate::dmy(.data$Date),
-      Venue = as.character(.data$Venue),
-      Team = as.character(.data$Team),
-      Score = as.character(.data$Score) %>% as.numeric(.),
-      Margin = as.character(.data$Margin) %>%
-        stringr::str_replace_all(., "\\+", "") %>%
-        as.numeric(.),
-      Win.Odds = as.character(.data$Win.Odds) %>% as.numeric(.),
-      Win.Paid = as.character(.data$Win.Paid) %>% as.numeric(.),
-      Line.Odds = as.character(.data$Line.Odds) %>%
-        stringr::str_replace_all(., "\\+", "") %>%
-        as.numeric(.),
-      Line.Paid = as.character(.data$Line.Paid) %>% as.numeric(.),
-      Season = as.character(.data$Season) %>% as.numeric(.),
-      # Raw betting data has two rows per match: the top team is home
-      # and the bottom is away
-      Home.Away = ifelse(dplyr::row_number() %% 2 == 1, "home", "away")
-    ) %>%
     tidyr::pivot_wider(
       .,
       names_from = c(.data$Home.Away),
@@ -572,11 +600,11 @@ get_footywire_betting_odds <- function(
         .data$Win.Odds, .data$Win.Paid, .data$Line.Odds, .data$Line.Paid
       )
     ) %>%
+    dplyr::select(., !c('Match.ID')) %>%
     dplyr::rename_if(
       ., names(.) %>% grepl("_home$|_away$", .),
       rename_home_away_columns
     ) %>%
-    calculate_round(.) %>%
     dplyr::mutate_at(c("Home.Team", "Away.Team"), replace_teams) %>%
     dplyr::mutate(Venue = replace_venues(.data$Venue)) %>%
     dplyr::filter(!is.na(.data$Date)) %>%
