@@ -23,10 +23,11 @@ test_that("get_afltables_player_ids works", {
   testthat::skip_if_offline()
   testthat::skip_on_cran()
   
-  expect_type(get_afltables_player_ids(2018), "list")
+  expect_type(get_afltables_player_ids(2020), "list")
   expect_type(get_afltables_player_ids(2017), "list")
-  expect_type(get_afltables_player_ids(c(2017, 2018, 2019)), "list")
+  expect_error(get_afltables_player_ids(c(2017, 2018, 2019)))
   expect_error(get_afltables_player_ids())
+  expect_error(get_afltables_player_ids(2018))
   expect_error(get_afltables_player_ids("a"))
 })
 
@@ -37,7 +38,7 @@ test_that("scape_afltables_ works", {
   
   url_new <- "https://afltables.com/afl/stats/games/2018/030820180812.html"
   url_old <- "https://afltables.com/afl/stats/games/1897/030618970508.html"
-  expect_type(scrape_afltables_match(url_new), "list")
+  expect_error(scrape_afltables_match(url_new))
   expect_type(scrape_afltables_match(url_old), "list")
   expect_error(scrape_afltables_match())
   expect_error(scrape_afltables_match(1))
